/*
 * Copyright 2010 Maurice Castro. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials provided
 *        with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Maurice Castro ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Maurice Castro OR OTHER CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <usb.h>

struct usb_device *opendev(u_int16_t idVendor, u_int16_t idProduct)
{
	struct usb_bus *busses;
	struct usb_bus *bus;
	struct usb_device *dev;
    
	busses = usb_get_busses();

	for (bus = busses; bus; bus = bus->next) 
	{
		for (dev = bus->devices; dev; dev = dev->next) 
		{
			if (dev->descriptor.idVendor == idVendor && dev->descriptor.idProduct == idProduct) 
			{
				/* Open the device, claim the interface and do your processing */
				return(dev);
			}
		}
	}
	return(NULL);
}

void erasedev(usb_dev_handle *devh)
{
	int8_t buf[] = {0xaa};
	if (usb_control_msg(devh, 0x40, 0x06, 0, 0, buf, 1, 0) < 0)
	{
		fprintf(stderr, "Failed to send control message\n");
		exit(1);
	}
}

int *readdev(usb_dev_handle *devh)
{
	int8_t buf[] = {0x55};
	int8_t buf2[128][8];
	int i;
	int *r;
	r = calloc(129, sizeof(r));
	r[128] = -1;
	if (!r)
	{
		fprintf(stderr, "Calloc failed\n");
		exit(1);
	}
	if (usb_control_msg(devh, 0x40, 0x06, 0, 0, buf, 1, 0) < 0)
	{
		fprintf(stderr, "Failed to send control message\n");
		exit(1);
	}
	for (i=0; i < 128; i++)
	{
		if (usb_interrupt_read(devh, 1, buf2[i], 8, 5000) < 0)
		{
			fprintf(stderr, "Read failed\n");
			exit(1);
		}
	}
	for (i = 0; i < 128; i++)
	{
		if (buf2[i][1] == -1 && buf2[i][0] == -1)
			r[i] = -1;
		else
			r[i] = (u_int8_t) buf2[i][1] * 256 + (u_int8_t) buf2[i][0];
	}
	return(r);
}

int main(int argc, char *argv[])
{
	struct usb_device *dev;
	usb_dev_handle *devh;
	int *a;

	usb_init();
	usb_find_busses();
	usb_find_devices();

	dev = opendev(0x04f3, 0xa003);
	if (!dev)
	{
		fprintf(stderr, "Failed to find device\n");
		exit(1);
	}
	devh = usb_open(dev);
	if (!devh)
	{
		fprintf(stderr, "Failed to open device\n");
		exit(1);
	}
	if (usb_set_configuration(devh, 1) < 0)
	{
		fprintf(stderr, "Failed to set configuration\n");
		exit(1);
	}
	if (usb_claim_interface(devh, 0) < 0)
	{
		fprintf(stderr, "Failed to claim interface\n");
		exit(1);
	}
	if (usb_set_altinterface(devh, 0) < 0)
	{
		fprintf(stderr, "Failed to set alternate interface\n");
		exit(1);
	}
	/* do the work */
	/* read device */
	a  = readdev(devh);
	while (*a > 0)
	{
		printf("%d\n", *a);
		a++;
	}
	/* erase device */
	erasedev(devh);
	/* close it all down */
	if (usb_release_interface(devh, 0) < 0)
	{
		fprintf(stderr, "Failed to release interface\n");
		exit(1);
	}
	if (usb_close(devh) < 0)
	{
		fprintf(stderr, "Failed to close device\n");
		exit(1);
	}
}

